# WARNING: Please don't edit this file. It was generated by Python/WinRT v2.3.0

import enum

import winrt.system
from winrt import _winrt_windows_devices_spi

__all__ = [
    "SpiMode",
    "SpiSharingMode",
    "SpiBusInfo",
    "SpiConnectionSettings",
    "SpiController",
    "SpiDevice",
    "ISpiDeviceStatics",
]

class SpiMode(enum.IntEnum):
    MODE0 = 0
    MODE1 = 1
    MODE2 = 2
    MODE3 = 3

class SpiSharingMode(enum.IntEnum):
    EXCLUSIVE = 0
    SHARED = 1

SpiBusInfo = _winrt_windows_devices_spi.SpiBusInfo
SpiConnectionSettings = _winrt_windows_devices_spi.SpiConnectionSettings
SpiController = _winrt_windows_devices_spi.SpiController
SpiDevice = _winrt_windows_devices_spi.SpiDevice
ISpiDeviceStatics = _winrt_windows_devices_spi.ISpiDeviceStatics
